//+------------------------------------------------------------------+
//| This script code was extracted from WHRea.mq4 which was authored 
//| by WHRoeder: WHRoeder@yahoo.com
//| and modified by global3
//+------------------------------------------------------------------+

#property show_inputs

//---- input parameters

extern int      MagicNumber = 100;

extern string  sep1 = "---------------------------------------------------------------------";

extern string  MM               = "---- Money Management Mode ----";
extern string  MMNote1 = "0=Fixed: Lot size as % of 1 lot ($100,000)";
extern string  MMNote2 = "1=Moderate: SQRT(Risk%*AB), 2=Geometrical: Risk%*AB";
extern string  MMNote3 = "AB=AccountEquity() - equityatrisk";
extern int      MMFix0Mod1Geo2 =   2;  // Money Management Mode

#define MMMODE_FIXED        0   // Lot size as a % of 1 lot ($100,000)
#define MMMODE_MODERATE     1   // SQRT(Risk%*AB)
#define MMMODE_GEOMETRICAL  2   // Risk%*AB

extern string  sep2 = "---------------------------------------------------------------------";

extern string   RSK  = "---- NewOrder, PerChart & Max Risks as % ----";					
extern double   MMNewOrderRisk = 2;
extern double   MMPerChartRisk =  3;
extern double   MMMaxRisk =  5;

extern string  sep3 = "---------------------------------------------------------------------";

extern string   SLIP              = "---- Max allowed slippage in pips ----";
extern double SlippagePips    =   3.0;

extern string  sep4 = "---------------------------------------------------------------------";

extern string   Lev               = "---- Max allowed real leverage ----";
extern double MaxLeverage   = 10.0;
extern double MarginCallLevel = 120; //As a percentage

extern string  sep5 = "---------------------------------------------------------------------";

extern string   TS               = "---- Tradesize in lots ----";
extern string   TS1             = "set to 0 to use autorisk";
extern double  tradesize      = 0.0;

extern string  sep6 = "---------------------------------------------------------------------";

extern string   SLTP               = "---- StopLoss & TakeProfit in pips ----";
extern double   takeprofit        = 60;
extern double   stoploss          = 30;

extern string  sep7 = "---------------------------------------------------------------------";
extern string   TEST             = "---- Test Script ----";
extern bool     Testing          = true;

//+-----------------------------------------------------------------+
//| Initialize variables for GetRisk function
//+-----------------------------------------------------------------+   

int         ooticket,  oocount;           // \  Import
double    ooprice,   equityatrisk,     //  \ from
            ooSL,      chartatrisk,      //  / ModifyStops
            ooTP,                          // /
            ooPO,                          // Import from SetStops
            ooOP,                          // Import from Decide
            balanceatrisk, //  Export to Lotsize
            oolots;
			
//+-----------------------------------------------------------------+
//| Initialize variables for SetDIR function
//+-----------------------------------------------------------------+            
            
string  optext;
double  nowopen,
        nowclose,
        opentoBid,
        stoptoBid,
        DIR;        // +/-1
int     opcode;    // OP_BUY/OP_SELL

int start()
{
//++++ These are adjusted for 5 digit brokers.
int     pips2points;    // slippage  3 pips    3=points    30=points
double  pips2dbl;       // Stoploss 15 pips    0.0015      0.00150
int     Digitspips;    // DoubleToStr(dbl/pips2dbl, Digitspips)

if (Digits == 5 || Digits == 3){    // Adjust for five (5) digit brokers.
                pips2dbl    = Point*10; pips2points = 10;   Digitspips = 1; }
    else {    pips2dbl    = Point;    pips2points =  1;   Digitspips = 0; }

    /*++++ Adjust for the current chart timeframe */{               static
    int     TFperiod[]  = { PERIOD_M1,  PERIOD_M5,  PERIOD_M15, PERIOD_M30,
                            PERIOD_H1, PERIOD_H4, PERIOD_D1, PERIOD_W1,
                            PERIOD_MN1 };                      static
    string  TFtext[]    = { "M1", "M5", "M15", "M30", "H1", "H4", "D1", "W1", "MN1" };
        for(int index=0; TFperiod[index] < Period(); index++){}
    string TF         = TFtext[index];
    MagicNumber    = MagicNumber + index;
    /*---- Adjust for the current chart timeframe */}

//Stoploss and Takeprofit as points
takeprofit  *= pips2dbl;
stoploss  *= pips2dbl;

//Use Money Management Mode to Calculate Lot size if tradesize = 0
   if (tradesize == 0){

double  atrisknew;
string pair;

double MarginCallLevel1 = MarginCallLevel * 0.01; 

////////////////////////////////////////////
   equityatrisk = 0; balanceatrisk = 0;  chartatrisk = 0;
  
    //Calculate chartatrisk, balanceatrisk and equityatrisk
    GetRisk();

    /*++++ Compute lot size based on account equity and MM mode*/
    double  ab  = AccountEquity() - equityatrisk;
    switch(MMFix0Mod1Geo2){
    case MMMODE_FIXED:                                              
	    double
        NewOrderRisk = MMNewOrderRisk * 0.01 * 100000,
        perChrt = MMPerChartRisk * 0.01 * 100000,
        maxRisk = MMMaxRisk * 0.01 * 100000;
        break;
    case MMMODE_MODERATE:
        // See http://articles.mql4.com/631 Fallacies, Part 1: Money
        // Management is Secondary and Not Very Important.
        NewOrderRisk = MathSqrt(MMNewOrderRisk * 0.01 * ab);
        perChrt = MathSqrt(MMPerChartRisk * 0.01 * ab);
        maxRisk = MathSqrt(MMMaxRisk * 0.01  * ab);
        break;
    case MMMODE_GEOMETRICAL:
        NewOrderRisk = MMNewOrderRisk * 0.01 * ab;
        perChrt = MMPerChartRisk * 0.01 * ab;
        maxRisk = MMMaxRisk * 0.01  * ab;
        break;
    }
    
double AccountLeverage1 = AccountLeverage();

double  minLot  = MarketInfo(Symbol(), MODE_MINLOT),
            tickvalue = MarketInfo(Symbol(), MODE_TICKVALUE),
            ticksize = MarketInfo(Symbol(), MODE_TICKSIZE),
            lotsize = MarketInfo(Symbol(), MODE_LOTSIZE),
            lotStep = MarketInfo(Symbol(), MODE_LOTSTEP),
            perLotPerPoint  =  DeltaValuePerLot(),
            maxLossPerLot   = (stoploss + (SlippagePips * pips2dbl)) * perLotPerPoint;
            tradesize = NewOrderRisk / maxLossPerLot;  // Must still round to lotStep.

while (true){   // Adjust for broker, 
        // Round to lotStep.
        tradesize = MathFloor(MathMax(0,tradesize)/lotStep)*lotStep;         
        atrisknew = tradesize * maxLossPerLot;             // Export for Comment
        if (tradesize < minLot){ atrisknew = 0; Alert("Lotsize too small: "+tradesize); return(0); }
        
		// For one pair, one TF
        if (atrisknew+chartatrisk > perChrt){
            tradesize = (perChrt-chartatrisk)/maxLossPerLot;   // Must still round to lotStep.
            continue;   }
			
        // For all charts
        if (atrisknew+balanceatrisk > maxRisk){
            tradesize = (maxRisk-balanceatrisk)/maxLossPerLot;  // Must still round to lotStep.
            continue;   }
 
        double  AFMC    = AccountFreeMarginCheck(Symbol(), OP_BUY, tradesize),
			eRisk   = equityatrisk + atrisknew;
				
        //Test for margin
	    //Note: Not sure if multiplying by the MarginCallLevel % here is correct to 
		//take the Margin Call Level % into account for avoiding a margin call.
		//If the MarginCallLevel is 100% then both lines of code would be correct
		//but if the MarginCallLevel is 120% would multiplying by the MarginCallLevel %
		//be a correct Margin Call test? If not, then what is the correct way to do this?
        //if (AFMC*0.99 <= eRisk){
        if (AFMC*0.99 <= eRisk*MarginCallLevel1){
            tradesize *= 0.95;    // Must still round to lotStep.
            continue;   }   // Prevent margin call if new trade goes against us.

        double AccountEquity1 = AccountEquity();
        //Real Leverage = Position Size / Account Size
        double RealLeverage = (tradesize * lotsize)/ (AccountEquity1 - eRisk);

        // Keep trade at or below Max Leverage.
        if (RealLeverage > MaxLeverage){
            tradesize *= 0.95;   // Must still round to lotStep.
            continue;   }
        
		//Margin Level if all orders get stopped out
        double MarginLevel = (AccountEquity1 - eRisk) /(eRisk/AccountLeverage1);
        break;
    } 
} //  if (tradesize == 0)

 if(Testing){
        Alert("AccLeverage: "+DoubleToStr(AccountLeverage1,1)+" ML: "+DoubleToStr(MarginLevel, 0)+"%\nRL: " +DoubleToStr(RealLeverage,2)+" Risk: $"+DoubleToStr(eRisk,2)+" TF: " +TF+ "\nRisk: $" + DoubleToStr(NewOrderRisk, 1) + " Calculated Lots: "+tradesize+"\nSL: "+DoubleToStr(stoploss/pips2dbl, Digitspips)+" TP: "+DoubleToStr(takeprofit/pips2dbl, Digitspips)+"\npips2dbl: "+pips2dbl+" pips2points: "+pips2points+" Digitspips: "+Digitspips+"\nLotstep: "+lotStep+" LOTSIZE: "+lotsize+"\nperLotPerPoint: "+perLotPerPoint+" tickvalue: "+tickvalue+" ticksize: "+ticksize); //alert the user
        return(0);
}

   return(0);
} // int start()

double DeltaValuePerLot(string pair=""){
    if (pair == "") pair = Symbol();
    return(  MarketInfo(pair, MODE_TICKVALUE) / MarketInfo(pair, MODE_TICKSIZE) ); // Not Point.
}

//+--------------------------------------------------------------------+
//| SetDIR for my chart TF/my orders. Adjust variables for trade direction.
//+--------------------------------------------------------------------+
void    SetDIR(int op){
    opcode = op;   if (op == OP_BUY){
        nowopen    = Ask;              opentoBid = Bid-Ask;  // Open at Ask.
        nowclose   = Bid;              stoptoBid = 0;        // Stop at Bid.
        DIR         = +1.;              optext     = "Buy";
    } else {
        nowopen    = Bid;              opentoBid = 0;        // Open at Bid.
        nowclose   = Ask;              stoptoBid = Bid-Ask;  // Stop at Ask.
        DIR         = -1.;              optext     = "Sell";
    }
}   // SetDIR

//+---------------------------------------------------------------------------------------+
//| Calculate chartatrisk, balanceatrisk and equityatrisk for all open and pending orders.
//+---------------------------------------------------------------------------------------+
void  GetRisk(){
		  double perLotPerPoint  =  DeltaValuePerLot();
        string  symbolChart     = Symbol();
        equityatrisk  = 0;            // \    Export to   All orders, all chrt
        balanceatrisk = 0;            //  >   to          \ Only orders
        chartatrisk   = 0;            // X    Lotsize.    / below BE.
        ooticket       = 0;            //  \   Export
        ooTP           = 0;            //   \  to
        ooSL           = 0;            //    > start
        ooprice        = 0;            //   /  for
        ooPO           = 0;            //  /   comment     Set by SetStops.
        oolots         = 0;            // X    display.    Summed for multiple
        oocount        = 0;            // _>               open orders.

        DIR             = 0;            // SetDIR if my chart/my orders.
        for(int pos = OrdersTotal()-1; pos >= 0 ; pos--) if (
        OrderSelect(pos, SELECT_BY_POS) ){
             if (OrderSymbol()      == symbolChart ){        // My pair
                SetDIR( OrderType() );
                if (OrderMagicNumber() == MagicNumber){    // My magic number
                    double size = OrderLots();          oolots += size;
                    ooticket   = OrderTicket();        oocount++;
                    ooTP       = OrderTakeProfit();
                    ooSL       = OrderStopLoss();  ooprice = OrderOpenPrice();

                    // Max loss to balance due to this chart
                    double  eRisk   = (ooprice - ooSL)*DIR;
                    if(eRisk>0.) chartatrisk += eRisk * size * perLotPerPoint;
                }   // My magic number (my time frame.)
            }   // My pair
            // All charts, not just my chart, no SetDIR here.
            double  DIRection = Direction( OrderType() );
            perLotPerPoint  =  DeltaValuePerLot(OrderSymbol());
            // Max loss to balance due to all charts
            eRisk   = (OrderOpenPrice()-OrderStopLoss()) * DIRection;
            if (eRisk>0.) balanceatrisk += eRisk * OrderLots()*perLotPerPoint;
            // Max loss to equity due to all charts
            eRisk   = (OrderClosePrice()-OrderStopLoss()) * DIRection;
            //equityatrisk += eRisk* OrderLots() * perLotPerPoint;
            if(eRisk>0.) equityatrisk += eRisk* OrderLots() * perLotPerPoint;
        }   // For OrderSelect
}   // GetRisk //ModifyStops


// Direction( OrderType() );
// OrderType() returns integer from 0..5 (see: http://docs.mql4.com/trading/OrderType
// and specifically http://docs.mql4.com/constants/trading)
// "% 2" is division remainder mathmatical operation (% also called the modulus operator). 
// Example using OrderType() is: 0%2=0, 1%2=1, 2%2=0, 3%2=1, 4%2=0, 5%2=1;  
// Notice how result can only ever be 0 or 1.  Now, by multiplying 0 or 1 by 2 we get 0 or 2, yes?
// The final part of return() statement is: 1 - 0 or 1 - 2, yes?
// Which resolves to 1-0=+1=long trade direction: +1  or  1-2=-1=short trade direction: -1
// Meaning BUY/LONG OrderType()'s always cause function to return +1  and 
// SELL/SHORT OrderType()'s always cause function to return -1
//+------------------------------------------------------------------+
double  Direction(int op_xxx){  return( 1. - 2. * (op_xxx%2) ); }
//+------------------------------------------------------------------+
